<?php

use CRM_Updaterecur_ExtensionUtil as E;
use Civi\Test\HeadlessInterface;
use Civi\Test\HookInterface;
use Civi\Test\TransactionalInterface;

require_once(__DIR__ . '/../../../Civi/UpdateRecurTraits.php');

/**
 * Tests the ContributionRecur.ChangeSubscriptions API method.
 *
 * Tips:
 *  - With HookInterface, you may implement CiviCRM hooks directly in the test class.
 *    Simply create corresponding functions (e.g. "hook_civicrm_post(...)" or similar).
 *  - With TransactionalInterface, any data changes made by setUp() or test****() functions will
 *    rollback automatically -- as long as you don't manipulate schema or truncate tables.
 *    If this test needs to manipulate schema or truncate tables, then either:
 *       a. Do all that using setupHeadless() and Civi\Test.
 *       b. Disable TransactionalInterface, and handle all setup/teardown yourself.
 *
 * @group headless
 */
class Civi_Api4_ContributionRecur_ChangeSubscriptionTest extends \PHPUnit\Framework\TestCase implements HeadlessInterface, HookInterface, TransactionalInterface {

  use \Civi\UpdateRecurTraits;

  public function setUpHeadless() {
    // Civi\Test has many helpers, like install(), uninstall(), sql(), and sqlFile().
    // See: https://docs.civicrm.org/dev/en/latest/testing/phpunit/#civitest
    return \Civi\Test::headless()
      ->install(['org.civicrm.search_kit', 'org.civicrm.afform', 'mjwshared'])
      ->installMe(__DIR__)
      ->apply();
  }

  public function setUp() :void {
    parent::setUp();
  }

  public function tearDown() :void {
    parent::tearDown();
  }

  /**
   * Example: Test that a version is returned.
   */
  public function testCreatingActivitiesWorks() {
    $this->createFixture();

    $result = \Civi\Api4\ContributionRecur::changeSubscriptions(FALSE)
      ->setWhere([['id', 'IN', array_values($this->recurs)]])
      ->setNewAmount('2')
      ->setReason('test change to 2 GBP')
      ->execute();

    $this->assertEquals(2, $result->metadata['scheduled']);
    $this->assertEquals(1, $result->metadata['skipped']);

    // Load activities.
    $activities = \Civi\Api4\Activity::get(FALSE)
      ->addJoin('ActivityContact AS target', 'INNER', ['target.record_type_id', '=', 3])
      ->addSelect(
        'activity_type_id:name',
        'activity_date_time',
        'status_id:name',
        'subject',
        'details',
        'updaterecur_activity_fields.updaterecur_recur_id',
        'updaterecur_activity_fields.updaterecur_old_amount',
        'updaterecur_activity_fields.updaterecur_new_amount',
        'target.contact_id'
      )
      ->addWhere('id', 'IN', $result->column('id'))
      ->execute()
      ->indexBy('target.contact_id')->getArrayCopy();

    $wilmaUpgrader = $activities[$this->contacts['WilmaUpgrader']];
    $this->assertEquals(2, $wilmaUpgrader['updaterecur_activity_fields.updaterecur_new_amount']);
    $this->assertEquals(1, $wilmaUpgrader['updaterecur_activity_fields.updaterecur_old_amount']);
    $this->assertEquals($this->recurs[$this->contacts['WilmaUpgrader']], $wilmaUpgrader['updaterecur_activity_fields.updaterecur_recur_id']);
    $this->assertEquals('test change to 2 GBP', $wilmaUpgrader['subject']);
    $this->assertEquals('Scheduled', $wilmaUpgrader['status_id:name']);
    $this->assertEquals('updaterecur_change', $wilmaUpgrader['activity_type_id:name']);
    $this->assertGreaterThan(time() + 60 * 28, strtotime($wilmaUpgrader['activity_date_time']));
    $this->assertLessThan(time() + 60 * 32, strtotime($wilmaUpgrader['activity_date_time']));
    $this->assertMatchesRegularExpression(
      '@[0-9- :]{19} Increase amount from 1 to 2 GBP. Activity Scheduled for processing from 30 mins’ time@s',
      $wilmaUpgrader['details']);

    $barneyDowngrader = $activities[$this->contacts['BarneyDowngrader']];
    $this->assertEquals(2, $barneyDowngrader['updaterecur_activity_fields.updaterecur_new_amount']);
    $this->assertEquals(3, $barneyDowngrader['updaterecur_activity_fields.updaterecur_old_amount']);
    $this->assertEquals($this->recurs[$this->contacts['BarneyDowngrader']], $barneyDowngrader['updaterecur_activity_fields.updaterecur_recur_id']);
    $this->assertEquals('test change to 2 GBP', $barneyDowngrader['subject']);
    $this->assertEquals('Scheduled', $barneyDowngrader['status_id:name']);
    $this->assertEquals('updaterecur_change', $barneyDowngrader['activity_type_id:name']);
    $this->assertGreaterThan(time() + 60 * 28, strtotime($barneyDowngrader['activity_date_time']));
    $this->assertLessThan(time() + 60 * 32, strtotime($barneyDowngrader['activity_date_time']));
    $this->assertMatchesRegularExpression(
      '@[0-9- :]{19} Decrease amount from 3 to 2 GBP. Activity Scheduled for processing from 30 mins’ time@s',
      $barneyDowngrader['details']);

    $emberMaintainer = $activities[$this->contacts['EmberMaintainer']];
    $this->assertEquals(2, $emberMaintainer['updaterecur_activity_fields.updaterecur_new_amount']);
    $this->assertEquals(2, $emberMaintainer['updaterecur_activity_fields.updaterecur_old_amount']);
    $this->assertEquals($this->recurs[$this->contacts['EmberMaintainer']], $emberMaintainer['updaterecur_activity_fields.updaterecur_recur_id']);
    $this->assertEquals('test change to 2 GBP', $emberMaintainer['subject']);
    $this->assertEquals('Cancelled', $emberMaintainer['status_id:name']);
    $this->assertEquals('updaterecur_change', $emberMaintainer['activity_type_id:name']);
    $this->assertGreaterThan(time() - 60 * 2, strtotime($emberMaintainer['activity_date_time']));
    $this->assertLessThan(time() + 1, strtotime($emberMaintainer['activity_date_time']));
    $this->assertMatchesRegularExpression(
      '@[0-9- :]{19} Change amount from 2 to 2 GBP. This activity was created in a Cancelled state because the requested new amount is the same as the existing amount@s',
      $emberMaintainer['details']
    );

    // Now try to do it again, we expect a fail because there's pending changes for Wilma, Barney.
    try {
      $result = \Civi\Api4\ContributionRecur::changeSubscriptions(FALSE)
        ->setWhere([['id', 'IN', array_values($this->recurs)]])
        ->setNewAmount('2')
        ->setReason('test change to 2 GBP')
        ->execute();
      $this->fail("Expected exception on trying to submit changes while changes are pending.");
    }
    catch (\CRM_Core_Exception $e) {
      $this->assertMatchesRegularExpression('/^There are changes pending .*\['
        . $this->recurs[$this->contacts['WilmaUpgrader']]
        . ','
        . $this->recurs[$this->contacts['BarneyDowngrader']]
        . ']/', $e->getMessage());
    }
  }

}
